@@ -1,4 +1,6 @@
 /*
+ * vi:se ts=8:
+ *
  * regcomp and regexec -- regsub and regerror are elsewhere
  *
  *	Copyright (c) 1986 by University of Toronto.
@@ -49,17 +51,21 @@
  * precedence is structured in regular expressions.  Serious changes in
  * regular-expression syntax might require a total rethink.
  */
+
+#include "jimautoconf.h"
+
+#if defined(JIM_REGEXP)
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "jim.h"
-#include "jimautoconf.h"
 #include "jimregexp.h"
 #include "utf8.h"
 
-#if !defined(HAVE_REGCOMP) || defined(JIM_REGEXP)
+/* An arbitrary limit, but this seems enough. Must be less than 1000. */
+#define REG_MAX_PAREN 100
 
 /*
  * Structure for regexp "program".  This is essentially a linear encoding
@@ -77,10 +83,7 @@
  * to the thing following the set of BRANCHes.)  The opcodes are:
  */
 
-/* This *MUST* be less than (255-20)/2=117 */
-#define REG_MAX_PAREN 100
-
-/* definition	number	opnd?	meaning */
+/* definition		number	opnd?	meaning */
 #define	END	0	/* no	End of program. */
 #define	BOL	1	/* no	Match "" at beginning of line. */
 #define	EOL	2	/* no	Match "" at end of line. */
@@ -91,21 +94,28 @@
 #define	BACK	7	/* no	Match "", "next" ptr points backward. */
 #define	EXACTLY	8	/* str	Match this string. */
 #define	NOTHING	9	/* no	Match empty string. */
-#define	REP		10	/* max,min	Match this (simple) thing [min,max] times. */
-#define	REPMIN	11	/* max,min	Match this (simple) thing [min,max] times, mininal match. */
+#define	REP	10	/* max,min	Match this (simple) thing [min,max] times. */
+#define	REPMIN	11	/* max,min	Match this (simple) thing [min,max] times, minimal match. */
 #define	REPX	12	/* max,min	Match this (complex) thing [min,max] times. */
 #define	REPXMIN	13	/* max,min	Match this (complex) thing [min,max] times, minimal match. */
+#define	BOLX	14	/* no	Match "" at beginning of input. */
+#define	EOLX	15	/* no	Match "" at end of input. */
+#define	WORDA	16	/* no	Match "" at wordchar, where prev is nonword */
+#define	WORDZ	17	/* no	Match "" at nonwordchar, where prev is word */
 
-#define	WORDA	15	/* no	Match "" at wordchar, where prev is nonword */
-#define	WORDZ	16	/* no	Match "" at nonwordchar, where prev is word */
-#define	OPEN	20	/* no	Mark this point in input as start of #n. */
+#define	OPENNC 	1000	/* no	Non-capturing parentheses - must be OPEN-1 */
+#define	OPEN   	1001	/* no	Mark this point in input as start of #n. */
 			/*	OPEN+1 is number 1, etc. */
-#define	CLOSE	(OPEN+REG_MAX_PAREN)	/* no	Analogous to OPEN. */
+
+/* must not be any other opts between OPEN and CLOSE */
+
+#define	CLOSENC	2000 	/* no	Non-capturing parentheses - must be CLOSE-1 */
+#define	CLOSE	2001 	/* no	Analogous to OPEN. */
 #define	CLOSE_END	(CLOSE+REG_MAX_PAREN)
 
 /*
- * The first byte of the regexp internal "program" is actually this magic
- * number; the start node begins in the second byte.
+ * The first word of the regexp internal "program" is actually this magic
+ * number; the start node begins in the second word.
  */
 #define	REG_MAGIC	0xFADED00D
 
@@ -123,23 +133,22 @@
  * BACK		Normal "next" pointers all implicitly point forward; BACK
  *		exists to make loop structures possible.
  *
- * STAR,PLUS	'?', and complex '*' and '+', are implemented as circular
- *		BRANCH structures using BACK.  Simple cases (one character
- *		per match) are implemented with STAR and PLUS for speed
- *		and to minimize recursive plunges.
+ * REP,REPX	Repeated matches ('?', '*', '+' and {min,max}) are implemented
+ *              as either simple repeats (REP) or complex repeats (REPX).
+ *              These opcodes include a "min" and "max" count after the opcode.
+ *		This is followed by a fourth "current count" word that is
+ *		only used by REPX, as it implements a recursive match.
+ *		REPMIN and REPXMIN are identical except they implement minimal repeats.
  *
  * OPEN,CLOSE	...are numbered at compile time.
  */
 
 /*
- * A node is one char of opcode followed by two chars of "next" pointer.
- * "Next" pointers are stored as two 8-bit pieces, high order first.  The
- * value is a positive offset from the opcode of the node containing it.
+ * A node is one word of opcode followed by one word of "next" pointer.
+ * The "next" pointer value is a positive offset from the opcode of the node
+ * containing it.
  * An operand, if any, simply follows the node.  (Note that much of the
  * code generation knows about this implicit relationship.)
- *
- * Using two bytes for the "next" pointer is vast overkill for most things,
- * but allows patterns to get big without disasters.
  */
 #define	OP(preg, p)	(preg->program[p])
 #define	NEXT(preg, p)	(preg->program[p + 1])
@@ -156,14 +165,14 @@
 
 #define	FAIL(R,M)	{ (R)->err = (M); return (M); }
 #define	ISMULT(c)	((c) == '*' || (c) == '+' || (c) == '?' || (c) == '{')
-#define	META	"^$.[()|?{+*"
+#define	META		"^$.[()|?{+*"
 
 /*
  * Flags to be passed up and down.
  */
-#define	HASWIDTH	01	/* Known never to match null string. */
-#define	SIMPLE		02	/* Simple enough to be STAR/PLUS operand. */
-#define	SPSTART		04	/* Starts with * or +. */
+#define	HASWIDTH	1	/* Known never to match null string. */
+#define	SIMPLE		2	/* Simple enough to be STAR/PLUS operand. */
+#define	SPSTART		4	/* Starts with * or +. */
 #define	WORST		0	/* Worst case. */
 
 #define MAX_REP_COUNT 1000000
@@ -179,9 +188,9 @@ static int regnode(regex_t *preg, int op );
 static int regnext(regex_t *preg, int p );
 static void regc(regex_t *preg, int b );
 static int reginsert(regex_t *preg, int op, int size, int opnd );
-static void regtail_(regex_t *preg, int p, int val, int line );
+static void regtail(regex_t *preg, int p, int val);
 static void regoptail(regex_t *preg, int p, int val );
-#define regtail(PREG, P, VAL) regtail_(PREG, P, VAL, __LINE__)
+static int regopsize(regex_t *preg, int p );
 
 static int reg_range_find(const int *string, int c);
 static const char *str_find(const char *string, int c, int nocase);
@@ -189,7 +198,7 @@ static int prefix_cmp(const int *prog, int proglen, const char *string, int noca
 
 /*#define DEBUG*/
 #ifdef DEBUG
-int regnarrate = 0;
+static int regnarrate = 0;
 static void regdump(regex_t *preg);
 static const char *regprop( int op );
 #endif
@@ -240,17 +249,12 @@ int regcomp(regex_t *preg, const char *exp, int cflags)
 	/* First pass: determine size, legality. */
 	preg->cflags = cflags;
 	preg->regparse = exp;
-	/* XXX: For now, start unallocated */
-	preg->program = NULL;
-	preg->proglen = 0;
 
-#if 1
 	/* Allocate space. */
 	preg->proglen = (strlen(exp) + 1) * 5;
 	preg->program = malloc(preg->proglen * sizeof(int));
 	if (preg->program == NULL)
 		FAIL(preg, REG_ERR_NOMEM);
-#endif
 
 	/* Note that since we store a magic value as the first item in the program,
 	 * program offsets will never be 0
@@ -333,7 +337,14 @@ static int reg(regex_t *preg, int paren /* Parenthesized? */, int *flagp )
 
 	/* Make an OPEN node, if parenthesized. */
 	if (paren) {
-		parno = ++preg->re_nsub;
+		if (preg->regparse[0] == '?' && preg->regparse[1] == ':') {
+			/* non-capturing paren */
+			preg->regparse += 2;
+			parno = -1;
+		}
+		else {
+			parno = ++preg->re_nsub;
+		}
 		ret = regnode(preg, OPEN+parno);
 	} else
 		ret = 0;
@@ -436,7 +447,6 @@ static int regpiece(regex_t *preg, int *flagp)
 	char op;
 	int next;
 	int flags;
-	int chain = 0;
 	int min;
 	int max;
 
@@ -467,6 +477,10 @@ static int regpiece(regex_t *preg, int *flagp)
 		if (*end == '}') {
 			max = min;
 		}
+		else if (*end == '\0') {
+			preg->err = REG_ERR_UNMATCHED_BRACES;
+			return 0;
+		}
 		else {
 			preg->regparse = end;
 			max = strtoul(preg->regparse + 1, &end, 10);
@@ -519,12 +533,12 @@ static int regpiece(regex_t *preg, int *flagp)
 		return 0;
 	}
 
-	return chain ? chain : ret;
+	return ret;
 }
 
 /**
  * Add all characters in the inclusive range between lower and upper.
- * 
+ *
  * Handles a swapped range (upper < lower).
  */
 static void reg_addrange(regex_t *preg, int lower, int upper)
@@ -550,7 +564,7 @@ static void reg_addrange_str(regex_t *preg, const char *str)
 
 /**
  * Extracts the next unicode char from utf8.
- * 
+ *
  * If 'upper' is set, converts the char to uppercase.
  */
 static int reg_utf8_tounicode_case(const char *s, int *uc, int upper)
@@ -564,7 +578,7 @@ static int reg_utf8_tounicode_case(const char *s, int *uc, int upper)
 
 /**
  * Converts a hex digit to decimal.
- * 
+ *
  * Returns -1 for an invalid hex digit.
  */
 static int hexdigitval(int c)
@@ -604,7 +618,7 @@ static int parse_hex(const char *s, int n, int *uc)
 
 /**
  * Call for chars after a backlash to decode the escape sequence.
- * 
+ *
  * Stores the result in *ch.
  *
  * Returns the number of bytes consumed.
@@ -625,7 +639,23 @@ static int reg_decode_escape(const char *s, int *ch)
 		case 't': *ch = '\t'; break;
 		case 'v': *ch = '\v'; break;
 		case 'u':
-			if ((n = parse_hex(s, 4, ch)) > 0) {
+			if (*s == '{') {
+				/* Expect \u{NNNN} */
+				n = parse_hex(s + 1, 6, ch);
+				if (n > 0 && s[n + 1] == '}' && *ch >= 0 && *ch <= 0x1fffff) {
+					s += n + 2;
+				}
+				else {
+					/* Invalid, so just treat as an escaped 'u' */
+					*ch = 'u';
+				}
+			}
+			else if ((n = parse_hex(s, 4, ch)) > 0) {
+				s += n;
+			}
+			break;
+		case 'U':
+			if ((n = parse_hex(s, 8, ch)) > 0) {
 				s += n;
 			}
 			break;
@@ -694,15 +724,38 @@ static int regatom(regex_t *preg, int *flagp)
 				int start;
 				int end;
 
+				enum {
+					CC_ALPHA, CC_ALNUM, CC_SPACE, CC_BLANK, CC_UPPER, CC_LOWER,
+					CC_DIGIT, CC_XDIGIT, CC_CNTRL, CC_GRAPH, CC_PRINT, CC_PUNCT,
+					CC_NUM
+				};
+				int cc;
+
 				pattern += reg_utf8_tounicode_case(pattern, &start, nocase);
 				if (start == '\\') {
+					/* First check for class shorthand escapes */
+					switch (*pattern) {
+						case 's':
+							pattern++;
+							cc = CC_SPACE;
+							goto cc_switch;
+						case 'd':
+							pattern++;
+							cc = CC_DIGIT;
+							goto cc_switch;
+						case 'w':
+							pattern++;
+							reg_addrange(preg, '_', '_');
+							cc = CC_ALNUM;
+							goto cc_switch;
+					}
 					pattern += reg_decode_escape(pattern, &start);
 					if (start == 0) {
 						preg->err = REG_ERR_NULL_CHAR;
 						return 0;
 					}
 				}
-				if (pattern[0] == '-' && pattern[1]) {
+				if (pattern[0] == '-' && pattern[1] && pattern[1] != ']') {
 					/* skip '-' */
 					pattern += utf8_tounicode(pattern, &end);
 					pattern += reg_utf8_tounicode_case(pattern, &end, nocase);
@@ -717,27 +770,68 @@ static int regatom(regex_t *preg, int *flagp)
 					reg_addrange(preg, start, end);
 					continue;
 				}
-				if (start == '[') {
-					if (strncmp(pattern, ":alpha:]", 8) == 0) {
-						if ((preg->cflags & REG_ICASE) == 0) {
-							reg_addrange(preg, 'a', 'z');
+				if (start == '[' && pattern[0] == ':') {
+					static const char *character_class[] = {
+						":alpha:", ":alnum:", ":space:", ":blank:", ":upper:", ":lower:",
+						":digit:", ":xdigit:", ":cntrl:", ":graph:", ":print:", ":punct:",
+					};
+
+					for (cc = 0; cc < CC_NUM; cc++) {
+						n = strlen(character_class[cc]);
+						if (strncmp(pattern, character_class[cc], n) == 0) {
+							/* Found a character class */
+							pattern += n + 1;
+							break;
 						}
-						reg_addrange(preg, 'A', 'Z');
-						pattern += 8;
-						continue;
 					}
-					if (strncmp(pattern, ":alnum:]", 8) == 0) {
-						if ((preg->cflags & REG_ICASE) == 0) {
-							reg_addrange(preg, 'a', 'z');
+					if (cc != CC_NUM) {
+cc_switch:
+						switch (cc) {
+							case CC_ALNUM:
+								reg_addrange(preg, '0', '9');
+								/* Fall through */
+							case CC_ALPHA:
+								if ((preg->cflags & REG_ICASE) == 0) {
+									reg_addrange(preg, 'a', 'z');
+								}
+								reg_addrange(preg, 'A', 'Z');
+								break;
+							case CC_SPACE:
+								reg_addrange_str(preg, " \t\r\n\f\v");
+								break;
+							case CC_BLANK:
+								reg_addrange_str(preg, " \t");
+								break;
+							case CC_UPPER:
+								reg_addrange(preg, 'A', 'Z');
+								break;
+							case CC_LOWER:
+								reg_addrange(preg, 'a', 'z');
+								break;
+							case CC_XDIGIT:
+								reg_addrange(preg, 'a', 'f');
+								reg_addrange(preg, 'A', 'F');
+								/* Fall through */
+							case CC_DIGIT:
+								reg_addrange(preg, '0', '9');
+								break;
+							case CC_CNTRL:
+								reg_addrange(preg, 0, 31);
+								reg_addrange(preg, 127, 127);
+								break;
+							case CC_PRINT:
+								reg_addrange(preg, ' ', '~');
+								break;
+							case CC_GRAPH:
+								reg_addrange(preg, '!', '~');
+								break;
+							case CC_PUNCT:
+								reg_addrange(preg, '!', '/');
+								reg_addrange(preg, ':', '@');
+								reg_addrange(preg, '[', '`');
+								reg_addrange(preg, '{', '~');
+								break;
 						}
-						reg_addrange(preg, 'A', 'Z');
-						reg_addrange(preg, '0', '9');
-						pattern += 8;
-						continue;
-					}
-					if (strncmp(pattern, ":space:]", 8) == 0) {
-						reg_addrange_str(preg, " \t\r\n\f\v");
-						pattern += 8;
 						continue;
 					}
 				}
@@ -772,10 +866,17 @@ static int regatom(regex_t *preg, int *flagp)
 		preg->err = REG_ERR_COUNT_FOLLOWS_NOTHING;
 		return 0;
 	case '\\':
-		switch (*preg->regparse++) {
+		ch = *preg->regparse++;
+		switch (ch) {
 		case '\0':
 			preg->err = REG_ERR_TRAILING_BACKSLASH;
 			return 0;
+		case 'A':
+			ret = regnode(preg, BOLX);
+			break;
+		case 'Z':
+			ret = regnode(preg, EOLX);
+			break;
 		case '<':
 		case 'm':
 			ret = regnode(preg, WORDA);
@@ -785,13 +886,15 @@ static int regatom(regex_t *preg, int *flagp)
 			ret = regnode(preg, WORDZ);
 			break;
 		case 'd':
-			ret = regnode(preg, ANYOF);
+		case 'D':
+			ret = regnode(preg, ch == 'd' ? ANYOF : ANYBUT);
 			reg_addrange(preg, '0', '9');
 			regc(preg, '\0');
 			*flagp |= HASWIDTH|SIMPLE;
 			break;
 		case 'w':
-			ret = regnode(preg, ANYOF);
+		case 'W':
+			ret = regnode(preg, ch == 'w' ? ANYOF : ANYBUT);
 			if ((preg->cflags & REG_ICASE) == 0) {
 				reg_addrange(preg, 'a', 'z');
 			}
@@ -802,7 +905,8 @@ static int regatom(regex_t *preg, int *flagp)
 			*flagp |= HASWIDTH|SIMPLE;
 			break;
 		case 's':
-			ret = regnode(preg, ANYOF);
+		case 'S':
+			ret = regnode(preg, ch == 's' ? ANYOF : ANYBUT);
 			reg_addrange_str(preg," \t\r\n\f\v");
 			regc(preg, '\0');
 			*flagp |= HASWIDTH|SIMPLE;
@@ -840,7 +944,7 @@ static int regatom(regex_t *preg, int *flagp)
 					/* Non-trailing backslash.
 					 * Is this a special escape, or a regular escape?
 					 */
-					if (strchr("<>mMwds", preg->regparse[n])) {
+					if (strchr("<>mMwWdDsSAZ", preg->regparse[n])) {
 						/* A special escape. All done with EXACTLY */
 						break;
 					}
@@ -906,6 +1010,7 @@ static int regnode(regex_t *preg, int op)
 {
 	reg_grow(preg, 2);
 
+	/* The OP followed by a next pointer */
 	preg->program[preg->p++] = op;
 	preg->program[preg->p++] = 0;
 
@@ -947,7 +1052,7 @@ static int reginsert(regex_t *preg, int op, int size, int opnd )
 /*
  - regtail - set the next-pointer at the end of a node chain
  */
-static void regtail_(regex_t *preg, int p, int val, int line )
+static void regtail(regex_t *preg, int p, int val)
 {
 	int scan;
 	int temp;
@@ -1021,16 +1126,13 @@ int regexec(regex_t  *preg,  const  char *string, size_t nmatch, regmatch_t pmat
 	preg->nmatch = nmatch;
 	preg->start = string;	/* All offsets are computed from here */
 
-	/* Must clear out the embedded repeat counts */
-	for (scan = OPERAND(1); scan != 0; scan = regnext(preg, scan)) {
-		switch (OP(preg, scan)) {
-		case REP:
-		case REPMIN:
-		case REPX:
-		case REPXMIN:
-			preg->program[scan + 4] = 0;
+	/* Must clear out the embedded repeat counts of REPX and REPXMIN opcodes */
+	for (scan = OPERAND(1); scan != 0; scan += regopsize(preg, scan)) {
+		int op = OP(preg, scan);
+		if (op == END)
 			break;
-		}
+		if (op == REPX || op == REPXMIN)
+			preg->program[scan + 4] = 0;
 	}
 
 	/* If there is a "must appear" string, look for it. */
@@ -1056,8 +1158,7 @@ int regexec(regex_t  *preg,  const  char *string, size_t nmatch, regmatch_t pmat
 			goto nextline;
 		}
 		while (1) {
-			int ret = regtry(preg, string);
-			if (ret) {
+			if (regtry(preg, string)) {
 				return REG_NOERROR;
 			}
 			if (*string) {
@@ -1093,7 +1194,10 @@ nextline:
 			if (*s == '\0') {
 				break;
 			}
-			s += utf8_charlen(*s);
+			else {
+				int c;
+				s += utf8_tounicode(s, &c);
+			}
 		}
 
 	/* Failure. */
@@ -1150,7 +1254,7 @@ static int prefix_cmp(const int *prog, int proglen, const char *string, int noca
 
 /**
  * Searchs for 'c' in the range 'range'.
- * 
+ *
  * Returns 1 if found, or 0 if not.
  */
 static int reg_range_find(const int *range, int c)
@@ -1167,7 +1271,7 @@ static int reg_range_find(const int *range, int c)
 
 /**
  * Search for the character 'c' in the utf-8 string 'string'.
- * 
+ *
  * If 'nocase' is set, the 'string' is assumed to be uppercase
  * and 'c' is converted to uppercase before matching.
  *
@@ -1193,7 +1297,7 @@ static const char *str_find(const char *string, int c, int nocase)
 
 /**
  * Returns true if 'ch' is an end-of-line char.
- * 
+ *
  * In REG_NEWLINE mode, \n is considered EOL in
  * addition to \0
  */
@@ -1328,6 +1432,7 @@ static int regmatch(regex_t *preg, int prog)
 {
 	int scan;	/* Current node. */
 	int next;		/* Next node. */
+	const char *save;
 
 	scan = prog;
 
@@ -1340,7 +1445,6 @@ static int regmatch(regex_t *preg, int prog)
 		int c;
 #ifdef DEBUG
 		if (regnarrate) {
-			//fprintf(stderr, "%s...\n", regprop(scan));
 			fprintf(stderr, "%3d: %s...\n", scan, regprop(OP(preg, scan)));	/* Where, what. */
 		}
 #endif
@@ -1348,9 +1452,21 @@ static int regmatch(regex_t *preg, int prog)
 		n = reg_utf8_tounicode_case(preg->reginput, &c, (preg->cflags & REG_ICASE));
 
 		switch (OP(preg, scan)) {
-		case BOL:
-			if (preg->reginput != preg->regbol)
+		case BOLX:
+			if ((preg->eflags & REG_NOTBOL)) {
 				return(0);
+			}
+			/* Fall through */
+		case BOL:
+			if (preg->reginput != preg->regbol) {
+				return(0);
+			}
+			break;
+		case EOLX:
+			if (c != 0) {
+				/* For EOLX, only match real end of line, not newline */
+				return 0;
+			}
 			break;
 		case EOL:
 			if (!reg_iseol(preg, c)) {
@@ -1417,23 +1533,20 @@ static int regmatch(regex_t *preg, int prog)
 			break;
 		case BACK:
 			break;
-		case BRANCH: {
-				const char *save;
-
-				if (OP(preg, next) != BRANCH)		/* No choice. */
-					next = OPERAND(scan);	/* Avoid recursion. */
-				else {
-					do {
-						save = preg->reginput;
-						if (regmatch(preg, OPERAND(scan))) {
-							return(1);
-						}
-						preg->reginput = save;
-						scan = regnext(preg, scan);
-					} while (scan != 0 && OP(preg, scan) == BRANCH);
-					return(0);
-					/* NOTREACHED */
-				}
+		case BRANCH:
+			if (OP(preg, next) != BRANCH)		/* No choice. */
+				next = OPERAND(scan);	/* Avoid recursion. */
+			else {
+				do {
+					save = preg->reginput;
+					if (regmatch(preg, OPERAND(scan))) {
+						return(1);
+					}
+					preg->reginput = save;
+					scan = regnext(preg, scan);
+				} while (scan != 0 && OP(preg, scan) == BRANCH);
+				return(0);
+				/* NOTREACHED */
 			}
 			break;
 		case REP:
@@ -1445,36 +1558,31 @@ static int regmatch(regex_t *preg, int prog)
 			return regmatchrepeat(preg, scan, OP(preg, scan) == REPXMIN);
 
 		case END:
-			return(1);	/* Success! */
-			break;
+			return 1;	/* Success! */
+
+		case OPENNC:
+		case CLOSENC:
+			return regmatch(preg, next);
+
 		default:
 			if (OP(preg, scan) >= OPEN+1 && OP(preg, scan) < CLOSE_END) {
-				const char *save;
-
 				save = preg->reginput;
-
 				if (regmatch(preg, next)) {
-					int no;
-					/*
-					 * Don't set startp if some later
-					 * invocation of the same parentheses
-					 * already has.
-					 */
 					if (OP(preg, scan) < CLOSE) {
-						no = OP(preg, scan) - OPEN;
+						int no = OP(preg, scan) - OPEN;
 						if (no < preg->nmatch && preg->pmatch[no].rm_so == -1) {
 							preg->pmatch[no].rm_so = save - preg->start;
 						}
 					}
 					else {
-						no = OP(preg, scan) - CLOSE;
+						int no = OP(preg, scan) - CLOSE;
 						if (no < preg->nmatch && preg->pmatch[no].rm_eo == -1) {
 							preg->pmatch[no].rm_eo = save - preg->start;
 						}
 					}
 					return(1);
-				} else
-					return(0);
+				}
+				return(0);
 			}
 			return REG_ERR_INTERNAL;
 		}
@@ -1504,10 +1612,9 @@ static int regrepeat(regex_t *preg, int p, int max)
 	opnd = OPERAND(p);
 	switch (OP(preg, p)) {
 	case ANY:
-		/* No need to handle utf8 specially here */
 		while (!reg_iseol(preg, *scan) && count < max) {
 			count++;
-			scan++;
+			scan += utf8_charlen(*scan);
 		}
 		break;
 	case EXACTLY:
@@ -1568,7 +1675,32 @@ static int regnext(regex_t *preg, int p )
 		return(p+offset);
 }
 
-#ifdef DEBUG
+/*
+ - regopsize - returns the size of opcode + operands at 'p' in words
+ */
+static int regopsize(regex_t *preg, int p )
+{
+	/* Almost all opcodes are 2 words, but some are more */
+	switch (OP(preg, p)) {
+		case REP:
+		case REPMIN:
+		case REPX:
+		case REPXMIN:
+			return 5;
+
+		case ANYOF:
+		case ANYBUT:
+		case EXACTLY: {
+			int s = p + 2;
+			while (preg->program[s++]) {
+			}
+			return s - p;
+		}
+	}
+	return 2;
+}
+
+#if defined(DEBUG) && !defined(JIM_BOOTSTRAP)
 
 /*
  - regdump - dump a regexp onto stdout in vaguely comprehensible form
@@ -1578,12 +1710,12 @@ static void regdump(regex_t *preg)
 	int s;
 	int op = EXACTLY;	/* Arbitrary non-END op. */
 	int next;
-	char buf[4];
+	char buf[MAX_UTF8_LEN + 1];
 
 	int i;
 	for (i = 1; i < preg->p; i++) {
-		printf("%02x ", preg->program[i]);
-		if (i % 16 == 15) {
+		printf("%02x ", (unsigned char)preg->program[i]);
+		if (i % 16 == 0) {
 			printf("\n");
 		}
 	}
@@ -1617,10 +1749,10 @@ static void regdump(regex_t *preg)
 			while (preg->program[s]) {
 				int len = preg->program[s++];
 				int first = preg->program[s++];
-				buf[utf8_fromunicode(buf, first)] = 0;
+				buf[utf8_getchars(buf, first)] = 0;
 				printf("%s", buf);
 				if (len > 1) {
-					buf[utf8_fromunicode(buf, first + len - 1)] = 0;
+					buf[utf8_getchars(buf, first + len - 1)] = 0;
 					printf("-%s", buf);
 				}
 			}
@@ -1630,7 +1762,7 @@ static void regdump(regex_t *preg)
 			/* Literal string, where present. */
 
 			while (preg->program[s]) {
-				buf[utf8_fromunicode(buf, preg->program[s])] = 0;
+				buf[utf8_getchars(buf, preg->program[s])] = 0;
 				printf("%s", buf);
 				s++;
 			}
@@ -1642,7 +1774,7 @@ static void regdump(regex_t *preg)
 	if (op == END) {
 		/* Header fields of interest. */
 		if (preg->regstart) {
-			buf[utf8_fromunicode(buf, preg->regstart)] = 0;
+			buf[utf8_getchars(buf, preg->regstart)] = 0;
 			printf("start '%s' ", buf);
 		}
 		if (preg->reganch)
@@ -1671,6 +1803,10 @@ static const char *regprop( int op )
 		return "BOL";
 	case EOL:
 		return "EOL";
+	case BOLX:
+		return "BOLX";
+	case EOLX:
+		return "EOLX";
 	case ANY:
 		return "ANY";
 	case ANYOF:
@@ -1699,6 +1835,10 @@ static const char *regprop( int op )
 		return "WORDA";
 	case WORDZ:
 		return "WORDZ";
+	case OPENNC:
+		return "OPEN";
+	case CLOSENC:
+		return "CLOSE";
 	default:
 		if (op >= OPEN && op < CLOSE) {
 			snprintf(buf, sizeof(buf), "OPEN%d", op-OPEN);
@@ -1712,7 +1852,7 @@ static const char *regprop( int op )
 		return(buf);
 	}
 }
-#endif
+#endif /* JIM_BOOTSTRAP */
 
 size_t regerror(int errcode, const regex_t *preg, char *errbuf,  size_t errbuf_size)
 {
