@@ -730,7 +730,7 @@ NAND_DEVICE_COMMAND_HANDLER(davinci_nand_device_command)
 		goto fail;
 	}
 
-	info = calloc(1, sizeof *info);
+	info = calloc(1, sizeof(*info));
 	if (info == NULL)
 		goto fail;
 
