@@ -4,6 +4,7 @@
  *   Copyright (C) 2008 by Spencer Oliver <spen@spen-soft.co.uk>           *
  *   Copyright (C) 2009 Zachary T Welch <zw@superlucidity.net>             *
  *   Copyright (C) 2010 by Antonio Borneo <borneo.antonio@gmail.com>       *
+ *   Copyright (C) 2017-2018 Tomas Vanek <vanekt@fbl.cz>                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -36,21 +37,23 @@
 
 static struct flash_bank *flash_banks;
 
-int flash_driver_erase(struct flash_bank *bank, int first, int last)
+int flash_driver_erase(struct flash_bank *bank, unsigned int first,
+		unsigned int last)
 {
 	int retval;
 
 	retval = bank->driver->erase(bank, first, last);
 	if (retval != ERROR_OK)
-		LOG_ERROR("failed erasing sectors %d to %d", first, last);
+		LOG_ERROR("failed erasing sectors %u to %u", first, last);
 
 	return retval;
 }
 
-int flash_driver_protect(struct flash_bank *bank, int set, int first, int last)
+int flash_driver_protect(struct flash_bank *bank, int set, unsigned int first,
+		unsigned int last)
 {
 	int retval;
-	int num_blocks;
+	unsigned int num_blocks;
 
 	if (bank->num_prot_blocks)
 		num_blocks = bank->num_prot_blocks;
@@ -59,7 +62,7 @@ int flash_driver_protect(struct flash_bank *bank, int set, int first, int last)
 
 
 	/* callers may not supply illegal parameters ... */
-	if (first < 0 || first > last || last >= num_blocks) {
+	if (first > last || last >= num_blocks) {
 		LOG_ERROR("illegal protection block range");
 		return ERROR_FAIL;
 	}
@@ -67,6 +70,11 @@ int flash_driver_protect(struct flash_bank *bank, int set, int first, int last)
 	/* force "set" to 0/1 */
 	set = !!set;
 
+	if (bank->driver->protect == NULL) {
+		LOG_ERROR("Flash protection is not supported.");
+		return ERROR_FLASH_OPER_UNSUPPORTED;
+	}
+
 	/* DANGER!
 	 *
 	 * We must not use any cached information about protection state!!!!
@@ -80,20 +88,21 @@ int flash_driver_protect(struct flash_bank *bank, int set, int first, int last)
 	 */
 	retval = bank->driver->protect(bank, set, first, last);
 	if (retval != ERROR_OK)
-		LOG_ERROR("failed setting protection for blocks %d to %d", first, last);
+		LOG_ERROR("failed setting protection for blocks %u to %u", first, last);
 
 	return retval;
 }
 
 int flash_driver_write(struct flash_bank *bank,
-	uint8_t *buffer, uint32_t offset, uint32_t count)
+	const uint8_t *buffer, uint32_t offset, uint32_t count)
 {
 	int retval;
 
 	retval = bank->driver->write(bank, buffer, offset, count);
 	if (retval != ERROR_OK) {
 		LOG_ERROR(
-			"error writing to flash at address 0x%08" PRIx32 " at offset 0x%8.8" PRIx32,
+			"error writing to flash at address " TARGET_ADDR_FMT
+			" at offset 0x%8.8" PRIx32,
 			bank->base,
 			offset);
 	}
@@ -111,7 +120,8 @@ int flash_driver_read(struct flash_bank *bank,
 	retval = bank->driver->read(bank, buffer, offset, count);
 	if (retval != ERROR_OK) {
 		LOG_ERROR(
-			"error reading to flash at address 0x%08" PRIx32 " at offset 0x%8.8" PRIx32,
+			"error reading to flash at address " TARGET_ADDR_FMT
+			" at offset 0x%8.8" PRIx32,
 			bank->base,
 			offset);
 	}
@@ -125,6 +135,43 @@ int default_flash_read(struct flash_bank *bank,
 	return target_read_buffer(bank->target, offset + bank->base, count, buffer);
 }
 
+int flash_driver_verify(struct flash_bank *bank,
+	const uint8_t *buffer, uint32_t offset, uint32_t count)
+{
+	int retval;
+
+	retval = bank->driver->verify ? bank->driver->verify(bank, buffer, offset, count) :
+		default_flash_verify(bank, buffer, offset, count);
+	if (retval != ERROR_OK) {
+		LOG_ERROR("verify failed in bank at " TARGET_ADDR_FMT " starting at 0x%8.8" PRIx32,
+			bank->base, offset);
+	}
+
+	return retval;
+}
+
+int default_flash_verify(struct flash_bank *bank,
+	const uint8_t *buffer, uint32_t offset, uint32_t count)
+{
+	uint32_t target_crc, image_crc;
+	int retval;
+
+	retval = image_calculate_checksum(buffer, count, &image_crc);
+	if (retval != ERROR_OK)
+		return retval;
+
+	retval = target_checksum_memory(bank->target, offset + bank->base, count, &target_crc);
+	if (retval != ERROR_OK)
+		return retval;
+
+	LOG_DEBUG("addr " TARGET_ADDR_FMT ", len 0x%08" PRIx32 ", crc 0x%08" PRIx32 " 0x%08" PRIx32,
+		offset + bank->base, count, ~image_crc, ~target_crc);
+	if (target_crc == image_crc)
+		return ERROR_OK;
+	else
+		return ERROR_FAIL;
+}
+
 void flash_bank_add(struct flash_bank *bank)
 {
 	/* put flash bank in linked list */
@@ -149,10 +196,10 @@ struct flash_bank *flash_bank_list(void)
 	return flash_banks;
 }
 
-struct flash_bank *get_flash_bank_by_num_noprobe(int num)
+struct flash_bank *get_flash_bank_by_num_noprobe(unsigned int num)
 {
 	struct flash_bank *p;
-	int i = 0;
+	unsigned int i = 0;
 
 	for (p = flash_banks; p; p = p->next) {
 		if (i++ == num)
@@ -162,15 +209,48 @@ struct flash_bank *get_flash_bank_by_num_noprobe(int num)
 	return NULL;
 }
 
-int flash_get_bank_count(void)
+unsigned int flash_get_bank_count(void)
 {
 	struct flash_bank *p;
-	int i = 0;
+	unsigned int i = 0;
 	for (p = flash_banks; p; p = p->next)
 		i++;
 	return i;
 }
 
+void default_flash_free_driver_priv(struct flash_bank *bank)
+{
+	free(bank->driver_priv);
+	bank->driver_priv = NULL;
+}
+
+void flash_free_all_banks(void)
+{
+	struct flash_bank *bank = flash_banks;
+	while (bank) {
+		struct flash_bank *next = bank->next;
+		if (bank->driver->free_driver_priv)
+			bank->driver->free_driver_priv(bank);
+		else
+			LOG_WARNING("Flash driver of %s does not support free_driver_priv()", bank->name);
+
+		/* For 'virtual' flash driver bank->sectors and bank->prot_blocks pointers are copied from
+		 * master flash_bank structure. They point to memory locations allocated by master flash driver
+		 * so master driver is responsible for releasing them.
+		 * Avoid UB caused by double-free memory corruption if flash bank is 'virtual'. */
+
+		if (strcmp(bank->driver->name, "virtual") != 0) {
+			free(bank->sectors);
+			free(bank->prot_blocks);
+		}
+
+		free(bank->name);
+		free(bank);
+		bank = next;
+	}
+	flash_banks = NULL;
+}
+
 struct flash_bank *get_flash_bank_by_name_noprobe(const char *name)
 {
 	unsigned requested = get_flash_name_index(name);
@@ -208,7 +288,7 @@ int get_flash_bank_by_name(const char *name, struct flash_bank **bank_result)
 	return ERROR_OK;
 }
 
-int get_flash_bank_by_num(int num, struct flash_bank **bank)
+int get_flash_bank_by_num(unsigned int num, struct flash_bank **bank)
 {
 	struct flash_bank *p = get_flash_bank_by_num_noprobe(num);
 	int retval;
@@ -229,7 +309,7 @@ int get_flash_bank_by_num(int num, struct flash_bank **bank)
 /* lookup flash bank by address, bank not found is success, but
  * result_bank is set to NULL. */
 int get_flash_bank_by_addr(struct target *target,
-	uint32_t addr,
+	target_addr_t addr,
 	bool check,
 	struct flash_bank **result_bank)
 {
@@ -255,7 +335,7 @@ int get_flash_bank_by_addr(struct target *target,
 	}
 	*result_bank = NULL;
 	if (check) {
-		LOG_ERROR("No flash at address 0x%08" PRIx32, addr);
+		LOG_ERROR("No flash at address " TARGET_ADDR_FMT, addr);
 		return ERROR_FAIL;
 	}
 	return ERROR_OK;
@@ -265,7 +345,6 @@ static int default_flash_mem_blank_check(struct flash_bank *bank)
 {
 	struct target *target = bank->target;
 	const int buffer_size = 1024;
-	int i;
 	uint32_t nBytes;
 	int retval = ERROR_OK;
 
@@ -276,15 +355,15 @@ static int default_flash_mem_blank_check(struct flash_bank *bank)
 
 	uint8_t *buffer = malloc(buffer_size);
 
-	for (i = 0; i < bank->num_sectors; i++) {
+	for (unsigned int i = 0; i < bank->num_sectors; i++) {
 		uint32_t j;
 		bank->sectors[i].is_erased = 1;
 
 		for (j = 0; j < bank->sectors[i].size; j += buffer_size) {
 			uint32_t chunk;
 			chunk = buffer_size;
-			if (chunk > (j - bank->sectors[i].size))
-				chunk = (j - bank->sectors[i].size);
+			if (chunk > (bank->sectors[i].size - j))
+				chunk = (bank->sectors[i].size - j);
 
 			retval = target_read_memory(target,
 					bank->base + bank->sectors[i].offset + j,
@@ -312,38 +391,50 @@ done:
 int default_flash_blank_check(struct flash_bank *bank)
 {
 	struct target *target = bank->target;
-	int i;
 	int retval;
-	int fast_check = 0;
-	uint32_t blank;
 
 	if (bank->target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
 
-	for (i = 0; i < bank->num_sectors; i++) {
-		uint32_t address = bank->base + bank->sectors[i].offset;
-		uint32_t size = bank->sectors[i].size;
+	struct target_memory_check_block *block_array;
+	block_array = malloc(bank->num_sectors * sizeof(struct target_memory_check_block));
+	if (block_array == NULL)
+		return default_flash_mem_blank_check(bank);
 
-		retval = target_blank_check_memory(target, address, size, &blank, bank->erased_value);
-		if (retval != ERROR_OK) {
-			fast_check = 0;
+	for (unsigned int i = 0; i < bank->num_sectors; i++) {
+		block_array[i].address = bank->base + bank->sectors[i].offset;
+		block_array[i].size = bank->sectors[i].size;
+		block_array[i].result = UINT32_MAX; /* erase state unknown */
+	}
+
+	bool fast_check = true;
+	for (unsigned int i = 0; i < bank->num_sectors; ) {
+		retval = target_blank_check_memory(target,
+				block_array + i, bank->num_sectors - i,
+				bank->erased_value);
+		if (retval < 1) {
+			/* Run slow fallback if the first run gives no result
+			 * otherwise use possibly incomplete results */
+			if (i == 0)
+				fast_check = false;
 			break;
 		}
-		if (blank == bank->erased_value)
-			bank->sectors[i].is_erased = 1;
-		else
-			bank->sectors[i].is_erased = 0;
-		fast_check = 1;
+		i += retval; /* add number of blocks done this round */
 	}
 
-	if (!fast_check) {
+	if (fast_check) {
+		for (unsigned int i = 0; i < bank->num_sectors; i++)
+			bank->sectors[i].is_erased = block_array[i].result;
+		retval = ERROR_OK;
+	} else {
 		LOG_USER("Running slow fallback erase check - add working memory");
-		return default_flash_mem_blank_check(bank);
+		retval = default_flash_mem_blank_check(bank);
 	}
+	free(block_array);
 
-	return ERROR_OK;
+	return retval;
 }
 
 /* Manipulate given flash region, selecting the bank according to target
@@ -362,13 +453,14 @@ int default_flash_blank_check(struct flash_bank *bank)
  * warning about those additions.
  */
 static int flash_iterate_address_range_inner(struct target *target,
-	char *pad_reason, uint32_t addr, uint32_t length,
+	char *pad_reason, target_addr_t addr, uint32_t length,
 	bool iterate_protect_blocks,
-	int (*callback)(struct flash_bank *bank, int first, int last))
+	int (*callback)(struct flash_bank *bank, unsigned int first,
+		unsigned int last))
 {
 	struct flash_bank *c;
 	struct flash_sector *block_array;
-	uint32_t last_addr = addr + length;	/* first address AFTER end */
+	target_addr_t last_addr = addr + length - 1;	/* the last address of range */
 	int first = -1;
 	int last = -1;
 	int i;
@@ -394,36 +486,37 @@ static int flash_iterate_address_range_inner(struct target *target,
 	}
 
 	/* check whether it all fits in this bank */
-	if (addr + length - 1 > c->base + c->size - 1) {
+	if (last_addr > c->base + c->size - 1) {
 		LOG_ERROR("Flash access does not fit into bank.");
 		return ERROR_FLASH_DST_BREAKS_ALIGNMENT;
 	}
 
-	addr -= c->base;
-	last_addr -= c->base;
+	if (c->prot_blocks == NULL || c->num_prot_blocks == 0) {
+		/* flash driver does not define protect blocks, use sectors instead */
+		iterate_protect_blocks = false;
+	}
 
-	if (iterate_protect_blocks && c->prot_blocks && c->num_prot_blocks) {
+	if (iterate_protect_blocks) {
 		block_array = c->prot_blocks;
 		num_blocks = c->num_prot_blocks;
 	} else {
 		block_array = c->sectors;
 		num_blocks = c->num_sectors;
-		iterate_protect_blocks = false;
 	}
 
-
 	for (i = 0; i < num_blocks; i++) {
 		struct flash_sector *f = &block_array[i];
-		uint32_t end = f->offset + f->size;
+		target_addr_t sector_addr = c->base + f->offset;
+		target_addr_t sector_last_addr = sector_addr + f->size - 1;
 
 		/* start only on a sector boundary */
 		if (first < 0) {
 			/* scanned past the first sector? */
-			if (addr < f->offset)
+			if (addr < sector_addr)
 				break;
 
 			/* is this the first sector? */
-			if (addr == f->offset)
+			if (addr == sector_addr)
 				first = i;
 
 			/* Does this need head-padding?  If so, pad and warn;
@@ -433,20 +526,20 @@ static int flash_iterate_address_range_inner(struct target *target,
 			 * ever know if that data was in use.  The warning
 			 * should help users sort out messes later.
 			 */
-			else if (addr < end && pad_reason) {
+			else if (addr <= sector_last_addr && pad_reason) {
 				/* FIXME say how many bytes (e.g. 80 KB) */
 				LOG_WARNING("Adding extra %s range, "
-					"%#8.8x to %#8.8x",
+					TARGET_ADDR_FMT " .. " TARGET_ADDR_FMT,
 					pad_reason,
-					(unsigned) f->offset,
-					(unsigned) addr - 1);
+					sector_addr,
+					addr - 1);
 				first = i;
 			} else
 				continue;
 		}
 
 		/* is this (also?) the last sector? */
-		if (last_addr == end) {
+		if (last_addr == sector_last_addr) {
 			last = i;
 			break;
 		}
@@ -454,28 +547,28 @@ static int flash_iterate_address_range_inner(struct target *target,
 		/* Does this need tail-padding?  If so, pad and warn;
 		 * or else force an error.
 		 */
-		if (last_addr < end && pad_reason) {
+		if (last_addr < sector_last_addr && pad_reason) {
 			/* FIXME say how many bytes (e.g. 80 KB) */
 			LOG_WARNING("Adding extra %s range, "
-				"%#8.8x to %#8.8x",
+				TARGET_ADDR_FMT " .. " TARGET_ADDR_FMT,
 				pad_reason,
-				(unsigned) last_addr,
-				(unsigned) end - 1);
+				last_addr + 1,
+				sector_last_addr);
 			last = i;
 			break;
 		}
 
 		/* MUST finish on a sector boundary */
-		if (last_addr <= f->offset)
+		if (last_addr < sector_addr)
 			break;
 	}
 
 	/* invalid start or end address? */
 	if (first == -1 || last == -1) {
-		LOG_ERROR("address range 0x%8.8x .. 0x%8.8x "
-			"is not sector-aligned",
-			(unsigned) (c->base + addr),
-			(unsigned) (c->base + last_addr - 1));
+		LOG_ERROR("address range " TARGET_ADDR_FMT " .. " TARGET_ADDR_FMT
+			" is not sector-aligned",
+			addr,
+			last_addr);
 		return ERROR_FLASH_DST_BREAKS_ALIGNMENT;
 	}
 
@@ -490,9 +583,10 @@ static int flash_iterate_address_range_inner(struct target *target,
  * multiple chips.
  */
 static int flash_iterate_address_range(struct target *target,
-	char *pad_reason, uint32_t addr, uint32_t length,
+	char *pad_reason, target_addr_t addr, uint32_t length,
 	bool iterate_protect_blocks,
-	int (*callback)(struct flash_bank *bank, int first, int last))
+	int (*callback)(struct flash_bank *bank, unsigned int first,
+		unsigned int last))
 {
 	struct flash_bank *c;
 	int retval = ERROR_OK;
@@ -524,18 +618,20 @@ static int flash_iterate_address_range(struct target *target,
 }
 
 int flash_erase_address_range(struct target *target,
-	bool pad, uint32_t addr, uint32_t length)
+	bool pad, target_addr_t addr, uint32_t length)
 {
 	return flash_iterate_address_range(target, pad ? "erase" : NULL,
 		addr, length, false, &flash_driver_erase);
 }
 
-static int flash_driver_unprotect(struct flash_bank *bank, int first, int last)
+static int flash_driver_unprotect(struct flash_bank *bank, unsigned int first,
+		unsigned int last)
 {
 	return flash_driver_protect(bank, 0, first, last);
 }
 
-int flash_unlock_address_range(struct target *target, uint32_t addr, uint32_t length)
+int flash_unlock_address_range(struct target *target, target_addr_t addr,
+		uint32_t length)
 {
 	/* By default, pad to sector boundaries ... the real issue here
 	 * is that our (only) caller *permanently* removes protection,
@@ -559,12 +655,91 @@ static int compare_section(const void *a, const void *b)
 		return -1;
 }
 
-int flash_write_unlock(struct target *target, struct image *image,
-	uint32_t *written, int erase, bool unlock)
+/**
+ * Get aligned start address of a flash write region
+ */
+target_addr_t flash_write_align_start(struct flash_bank *bank, target_addr_t addr)
+{
+	if (addr < bank->base || addr >= bank->base + bank->size
+			|| bank->write_start_alignment <= 1)
+		return addr;
+
+	if (bank->write_start_alignment == FLASH_WRITE_ALIGN_SECTOR) {
+		uint32_t offset = addr - bank->base;
+		uint32_t aligned = 0;
+		for (unsigned int sect = 0; sect < bank->num_sectors; sect++) {
+			if (bank->sectors[sect].offset > offset)
+				break;
+
+			aligned = bank->sectors[sect].offset;
+		}
+		return bank->base + aligned;
+	}
+
+	return addr & ~(bank->write_start_alignment - 1);
+}
+
+/**
+ * Get aligned end address of a flash write region
+ */
+target_addr_t flash_write_align_end(struct flash_bank *bank, target_addr_t addr)
+{
+	if (addr < bank->base || addr >= bank->base + bank->size
+			|| bank->write_end_alignment <= 1)
+		return addr;
+
+	if (bank->write_end_alignment == FLASH_WRITE_ALIGN_SECTOR) {
+		uint32_t offset = addr - bank->base;
+		uint32_t aligned = 0;
+		for (unsigned int sect = 0; sect < bank->num_sectors; sect++) {
+			aligned = bank->sectors[sect].offset + bank->sectors[sect].size - 1;
+			if (aligned >= offset)
+				break;
+		}
+		return bank->base + aligned;
+	}
+
+	return addr | (bank->write_end_alignment - 1);
+}
+
+/**
+ * Check if gap between sections is bigger than minimum required to discontinue flash write
+ */
+static bool flash_write_check_gap(struct flash_bank *bank,
+				target_addr_t addr1, target_addr_t addr2)
+{
+	if (bank->minimal_write_gap == FLASH_WRITE_CONTINUOUS
+			|| addr1 < bank->base || addr1 >= bank->base + bank->size
+			|| addr2 < bank->base || addr2 >= bank->base + bank->size)
+		return false;
+
+	if (bank->minimal_write_gap == FLASH_WRITE_GAP_SECTOR) {
+		unsigned int sect;
+		uint32_t offset1 = addr1 - bank->base;
+		/* find the sector following the one containing addr1 */
+		for (sect = 0; sect < bank->num_sectors; sect++) {
+			if (bank->sectors[sect].offset > offset1)
+				break;
+		}
+		if (sect >= bank->num_sectors)
+			return false;
+
+		uint32_t offset2 = addr2 - bank->base;
+		return bank->sectors[sect].offset + bank->sectors[sect].size <= offset2;
+	}
+
+	target_addr_t aligned1 = flash_write_align_end(bank, addr1);
+	target_addr_t aligned2 = flash_write_align_start(bank, addr2);
+	return aligned1 + bank->minimal_write_gap < aligned2;
+}
+
+
+int flash_write_unlock_verify(struct target *target, struct image *image,
+	uint32_t *written, bool erase, bool unlock, bool write, bool verify)
 {
 	int retval = ERROR_OK;
 
-	int section;
+	unsigned int section;
 	uint32_t section_offset;
 	struct flash_bank *c;
 	int *padding;
@@ -589,8 +764,8 @@ int flash_write_unlock(struct target *target, struct image *image,
 	 * whereas an image can have sections out of order. */
 	struct imagesection **sections = malloc(sizeof(struct imagesection *) *
 			image->num_sections);
-	int i;
-	for (i = 0; i < image->num_sections; i++)
+
+	for (unsigned int i = 0; i < image->num_sections; i++)
 		sections[i] = &image->sections[i];
 
 	qsort(sections, image->num_sections, sizeof(struct imagesection *),
@@ -598,10 +773,10 @@ int flash_write_unlock(struct target *target, struct image *image,
 
 	/* loop until we reach end of the image */
 	while (section < image->num_sections) {
-		uint32_t buffer_size;
+		uint32_t buffer_idx;
 		uint8_t *buffer;
-		int section_last;
-		uint32_t run_address = sections[section]->base_address + section_offset;
+		unsigned int section_last;
+		target_addr_t run_address = sections[section]->base_address + section_offset;
 		uint32_t run_size = sections[section]->size - section_offset;
 		int pad_bytes = 0;
 
@@ -617,7 +792,7 @@ int flash_write_unlock(struct target *target, struct image *image,
 		if (retval != ERROR_OK)
 			goto done;
 		if (c == NULL) {
-			LOG_WARNING("no flash bank found for address %" PRIx32, run_address);
+			LOG_WARNING("no flash bank found for address " TARGET_ADDR_FMT, run_address);
 			section++;	/* and skip it */
 			section_offset = 0;
 			continue;
@@ -635,32 +810,37 @@ int flash_write_unlock(struct target *target, struct image *image,
 				break;
 			}
 
-			/* FIXME This needlessly touches sectors BETWEEN the
-			 * sections it's writing.  Without auto erase, it just
-			 * writes ones.  That WILL INVALIDATE data in cases
-			 * like Stellaris Tempest chips, corrupting internal
-			 * ECC codes; and at least FreeScale suggests issues
-			 * with that approach (in HC11 documentation).
-			 *
-			 * With auto erase enabled, data in those sectors will
-			 * be needlessly destroyed; and some of the limited
-			 * number of flash erase cycles will be wasted...
-			 *
-			 * In both cases, the extra writes slow things down.
-			 */
-
 			/* if we have multiple sections within our image,
 			 * flash programming could fail due to alignment issues
 			 * attempt to rebuild a consecutive buffer for the flash loader */
-			pad_bytes = (sections[section_last + 1]->base_address) - (run_address + run_size);
-			padding[section_last] = pad_bytes;
-			run_size += sections[++section_last]->size;
-			run_size += pad_bytes;
+			target_addr_t run_next_addr = run_address + run_size;
+			target_addr_t next_section_base = sections[section_last + 1]->base_address;
+			if (next_section_base < run_next_addr) {
+				LOG_ERROR("Section at " TARGET_ADDR_FMT
+					" overlaps section ending at " TARGET_ADDR_FMT,
+					next_section_base, run_next_addr);
+				LOG_ERROR("Flash write aborted.");
+				retval = ERROR_FAIL;
+				goto done;
+			}
 
+			pad_bytes = next_section_base - run_next_addr;
+			if (pad_bytes) {
+				if (flash_write_check_gap(c, run_next_addr - 1, next_section_base)) {
+					LOG_INFO("Flash write discontinued at " TARGET_ADDR_FMT
+						", next section at " TARGET_ADDR_FMT,
+						run_next_addr, next_section_base);
+					break;
+				}
+			}
 			if (pad_bytes > 0)
-				LOG_INFO("Padding image section %d with %d bytes",
-					section_last-1,
-					pad_bytes);
+				LOG_INFO("Padding image section %d at " TARGET_ADDR_FMT
+					" with %d bytes",
+					section_last, run_next_addr, pad_bytes);
+
+			padding[section_last] = pad_bytes;
+			run_size += pad_bytes;
+			run_size += sections[++section_last]->size;
 		}
 
 		if (run_address + run_size - 1 > c->base + c->size - 1) {
@@ -673,16 +853,43 @@ int flash_write_unlock(struct target *target, struct image *image,
 			assert(run_size > 0);
 		}
 
-		/* If we're applying any sector automagic, then pad this
-		 * (maybe-combined) segment to the end of its last sector.
-		 */
-		if (unlock || erase) {
-			int sector;
+		uint32_t padding_at_start = 0;
+		if (c->write_start_alignment || c->write_end_alignment) {
+			/* align write region according to bank requirements */
+			target_addr_t aligned_start = flash_write_align_start(c, run_address);
+			padding_at_start = run_address - aligned_start;
+			if (padding_at_start > 0) {
+				LOG_WARNING("Section start address " TARGET_ADDR_FMT
+					" breaks the required alignment of flash bank %s",
+					run_address, c->name);
+				LOG_WARNING("Padding %" PRIu32 " bytes from " TARGET_ADDR_FMT,
+					padding_at_start, aligned_start);
+
+				run_address -= padding_at_start;
+				run_size += padding_at_start;
+			}
+
+			target_addr_t run_end = run_address + run_size - 1;
+			target_addr_t aligned_end = flash_write_align_end(c, run_end);
+			pad_bytes = aligned_end - run_end;
+			if (pad_bytes > 0) {
+				LOG_INFO("Padding image section %d at " TARGET_ADDR_FMT
+					" with %d bytes (bank write end alignment)",
+					section_last, run_end + 1, pad_bytes);
+
+				padding[section_last] += pad_bytes;
+				run_size += pad_bytes;
+			}
+
+		} else if (unlock || erase) {
+			/* If we're applying any sector automagic, then pad this
+			 * (maybe-combined) segment to the end of its last sector.
+			 */
 			uint32_t offset_start = run_address - c->base;
 			uint32_t offset_end = offset_start + run_size;
 			uint32_t end = offset_end, delta;
 
-			for (sector = 0; sector < c->num_sectors; sector++) {
+			for (unsigned int sector = 0; sector < c->num_sectors; sector++) {
 				end = c->sectors[sector].offset
 					+ c->sectors[sector].size;
 				if (offset_end <= end)
@@ -701,13 +908,17 @@ int flash_write_unlock(struct target *target, struct image *image,
 			retval = ERROR_FAIL;
 			goto done;
 		}
-		buffer_size = 0;
+
+		if (padding_at_start)
+			memset(buffer, c->default_padded_value, padding_at_start);
+
+		buffer_idx = padding_at_start;
 
 		/* read sections to the buffer */
-		while (buffer_size < run_size) {
+		while (buffer_idx < run_size) {
 			size_t size_read;
 
-			size_read = run_size - buffer_size;
+			size_read = run_size - buffer_idx;
 			if (size_read > sections[section]->size - section_offset)
 				size_read = sections[section]->size - section_offset;
 
@@ -720,23 +931,25 @@ int flash_write_unlock(struct target *target, struct image *image,
 			int t_section_num = diff / sizeof(struct imagesection);
 
 			LOG_DEBUG("image_read_section: section = %d, t_section_num = %d, "
-					"section_offset = %d, buffer_size = %d, size_read = %d",
-				(int)section, (int)t_section_num, (int)section_offset,
-				(int)buffer_size, (int)size_read);
+					"section_offset = %"PRIu32", buffer_idx = %"PRIu32", size_read = %zu",
+				section, t_section_num, section_offset,
+				buffer_idx, size_read);
 			retval = image_read_section(image, t_section_num, section_offset,
-					size_read, buffer + buffer_size, &size_read);
+					size_read, buffer + buffer_idx, &size_read);
 			if (retval != ERROR_OK || size_read == 0) {
 				free(buffer);
 				goto done;
 			}
 
-			/* see if we need to pad the section */
-			while (padding[section]--)
-				(buffer + buffer_size)[size_read++] = c->default_padded_value;
-
-			buffer_size += size_read;
+			buffer_idx += size_read;
 			section_offset += size_read;
 
+			/* see if we need to pad the section */
+			if (padding[section]) {
+				memset(buffer + buffer_idx, c->default_padded_value, padding[section]);
+				buffer_idx += padding[section];
+			}
+
 			if (section_offset >= sections[section]->size) {
 				section++;
 				section_offset = 0;
@@ -756,8 +969,17 @@ int flash_write_unlock(struct target *target, struct image *image,
 		}
 
 		if (retval == ERROR_OK) {
-			/* write flash sectors */
-			retval = flash_driver_write(c, buffer, run_address - c->base, run_size);
+			if (write) {
+				/* write flash sectors */
+				retval = flash_driver_write(c, buffer, run_address - c->base, run_size);
+			}
+		}
+
+		if (retval == ERROR_OK) {
+			if (verify) {
+				/* verify flash sectors */
+				retval = flash_driver_verify(c, buffer, run_address - c->base, run_size);
+			}
 		}
 
 		free(buffer);
@@ -779,20 +1001,19 @@ done:
 }
 
 int flash_write(struct target *target, struct image *image,
-	uint32_t *written, int erase)
+	uint32_t *written, bool erase)
 {
-	return flash_write_unlock(target, image, written, erase, false);
+	return flash_write_unlock_verify(target, image, written, erase, false, true, false);
 }
 
-struct flash_sector *alloc_block_array(uint32_t offset, uint32_t size, int num_blocks)
+struct flash_sector *alloc_block_array(uint32_t offset, uint32_t size,
+		unsigned int num_blocks)
 {
-	int i;
-
 	struct flash_sector *array = calloc(num_blocks, sizeof(struct flash_sector));
 	if (array == NULL)
 		return NULL;
 
-	for (i = 0; i < num_blocks; i++) {
+	for (unsigned int i = 0; i < num_blocks; i++) {
 		array[i].offset = offset;
 		array[i].size = size;
 		array[i].is_erased = -1;
