@@ -47,7 +47,7 @@
 #define SPIFI_INIT_STACK_SIZE  512
 
 struct lpcspifi_flash_bank {
-	int probed;
+	bool probed;
 	uint32_t ssp_base;
 	uint32_t io_base;
 	uint32_t ioconfig_base;
@@ -72,7 +72,7 @@ FLASH_BANK_COMMAND_HANDLER(lpcspifi_flash_bank_command)
 	}
 
 	bank->driver_priv = lpcspifi_info;
-	lpcspifi_info->probed = 0;
+	lpcspifi_info->probed = false;
 
 	return ERROR_OK;
 }
@@ -177,8 +177,8 @@ static int lpcspifi_set_hw_mode(struct flash_bank *bank)
 	retval = target_alloc_working_area(target, sizeof(spifi_init_code)
 		+ SPIFI_INIT_STACK_SIZE, &spifi_init_algorithm);
 	if (retval != ERROR_OK) {
-		LOG_ERROR("Insufficient working area to initialize SPIFI "\
-			"module. You must allocate at least %zdB of working "\
+		LOG_ERROR("Insufficient working area to initialize SPIFI "
+			"module. You must allocate at least %zdB of working "
 			"area in order to use this driver.",
 			sizeof(spifi_init_code) + SPIFI_INIT_STACK_SIZE
 		);
@@ -186,7 +186,7 @@ static int lpcspifi_set_hw_mode(struct flash_bank *bank)
 		return retval;
 	}
 
-	LOG_DEBUG("Writing algorithm to working area at 0x%08" PRIx32,
+	LOG_DEBUG("Writing algorithm to working area at " TARGET_ADDR_FMT,
 		spifi_init_algorithm->address);
 	/* Write algorithm to working area */
 	retval = target_write_buffer(target,
@@ -216,7 +216,7 @@ static int lpcspifi_set_hw_mode(struct flash_bank *bank)
 
 	/* Run the algorithm */
 	LOG_DEBUG("Running SPIFI init algorithm");
-	retval = target_run_algorithm(target, 0 , NULL, 2, reg_params,
+	retval = target_run_algorithm(target, 0, NULL, 2, reg_params,
 		spifi_init_algorithm->address,
 		spifi_init_algorithm->address + sizeof(spifi_init_code) - 2,
 		1000, &armv7m_info);
@@ -387,6 +387,9 @@ static int lpcspifi_bulk_erase(struct flash_bank *bank)
 	uint32_t value;
 	int retval = ERROR_OK;
 
+	if (lpcspifi_info->dev->chip_erase_cmd == 0x00)
+		return ERROR_FLASH_OPER_UNSUPPORTED;
+
 	retval = lpcspifi_set_sw_mode(bank);
 
 	if (retval == ERROR_OK)
@@ -411,7 +414,8 @@ static int lpcspifi_bulk_erase(struct flash_bank *bank)
 	return retval;
 }
 
-static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
+static int lpcspifi_erase(struct flash_bank *bank, unsigned int first,
+		unsigned int last)
 {
 	struct target *target = bank->target;
 	struct lpcspifi_flash_bank *lpcspifi_info = bank->driver_priv;
@@ -419,16 +423,15 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 	struct armv7m_algorithm armv7m_info;
 	struct working_area *erase_algorithm;
 	int retval = ERROR_OK;
-	int sector;
 
-	LOG_DEBUG("erase from sector %d to sector %d", first, last);
+	LOG_DEBUG("erase from sector %u to sector %u", first, last);
 
 	if (target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_NOT_HALTED;
 	}
 
-	if ((first < 0) || (last < first) || (last >= bank->num_sectors)) {
+	if ((last < first) || (last >= bank->num_sectors)) {
 		LOG_ERROR("Flash sector invalid");
 		return ERROR_FLASH_SECTOR_INVALID;
 	}
@@ -438,9 +441,9 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 		return ERROR_FLASH_BANK_NOT_PROBED;
 	}
 
-	for (sector = first; sector <= last; sector++) {
+	for (unsigned int sector = first; sector <= last; sector++) {
 		if (bank->sectors[sector].is_protected) {
-			LOG_ERROR("Flash sector %d protected", sector);
+			LOG_ERROR("Flash sector %u protected", sector);
 			return ERROR_FAIL;
 		}
 	}
@@ -449,7 +452,7 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 	 * it, use a bulk erase instead of going sector-by-sector. */
 	if (first == 0 && last == (bank->num_sectors - 1)
 		&& lpcspifi_info->dev->chip_erase_cmd != lpcspifi_info->dev->erase_cmd) {
-		LOG_DEBUG("Chip supports the bulk erase command."\
+		LOG_DEBUG("Chip supports the bulk erase command."
 		" Will use bulk erase instead of sector-by-sector erase.");
 		retval = lpcspifi_bulk_erase(bank);
 
@@ -460,6 +463,9 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 			LOG_WARNING("Bulk flash erase failed. Falling back to sector-by-sector erase.");
 	}
 
+	if (lpcspifi_info->dev->erase_cmd == 0x00)
+		return ERROR_FLASH_OPER_UNSUPPORTED;
+
 	retval = lpcspifi_set_hw_mode(bank);
 	if (retval != ERROR_OK)
 		return retval;
@@ -519,7 +525,7 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 	retval = target_alloc_working_area(target, sizeof(lpcspifi_flash_erase_code),
 		&erase_algorithm);
 	if (retval != ERROR_OK) {
-		LOG_ERROR("Insufficient working area. You must configure a working"\
+		LOG_ERROR("Insufficient working area. You must configure a working"
 			" area of at least %zdB in order to erase SPIFI flash.",
 			sizeof(lpcspifi_flash_erase_code));
 		return retval;
@@ -544,7 +550,7 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 	buf_set_u32(reg_params[3].value, 0, 32, bank->sectors[first].size);
 
 	/* Run the algorithm */
-	retval = target_run_algorithm(target, 0 , NULL, 4, reg_params,
+	retval = target_run_algorithm(target, 0, NULL, 4, reg_params,
 		erase_algorithm->address,
 		erase_algorithm->address + sizeof(lpcspifi_flash_erase_code) - 4,
 		3000*(last - first + 1), &armv7m_info);
@@ -565,11 +571,9 @@ static int lpcspifi_erase(struct flash_bank *bank, int first, int last)
 }
 
 static int lpcspifi_protect(struct flash_bank *bank, int set,
-	int first, int last)
+	unsigned int first, unsigned int last)
 {
-	int sector;
-
-	for (sector = first; sector <= last; sector++)
+	for (unsigned int sector = first; sector <= last; sector++)
 		bank->sectors[sector].is_protected = set;
 	return ERROR_OK;
 }
@@ -584,7 +588,6 @@ static int lpcspifi_write(struct flash_bank *bank, const uint8_t *buffer,
 	struct reg_param reg_params[5];
 	struct armv7m_algorithm armv7m_info;
 	struct working_area *write_algorithm;
-	int sector;
 	int retval = ERROR_OK;
 
 	LOG_DEBUG("offset=0x%08" PRIx32 " count=0x%08" PRIx32,
@@ -601,19 +604,21 @@ static int lpcspifi_write(struct flash_bank *bank, const uint8_t *buffer,
 	}
 
 	/* Check sector protection */
-	for (sector = 0; sector < bank->num_sectors; sector++) {
+	for (unsigned int sector = 0; sector < bank->num_sectors; sector++) {
 		/* Start offset in or before this sector? */
 		/* End offset in or behind this sector? */
 		if ((offset <
 				(bank->sectors[sector].offset + bank->sectors[sector].size))
 			&& ((offset + count - 1) >= bank->sectors[sector].offset)
 			&& bank->sectors[sector].is_protected) {
-			LOG_ERROR("Flash sector %d protected", sector);
+			LOG_ERROR("Flash sector %u protected", sector);
 			return ERROR_FAIL;
 		}
 	}
 
-	page_size = lpcspifi_info->dev->pagesize;
+	/* if no valid page_size, use reasonable default */
+	page_size = lpcspifi_info->dev->pagesize ?
+		lpcspifi_info->dev->pagesize : SPIFLASH_DEF_PAGESIZE;
 
 	retval = lpcspifi_set_hw_mode(bank);
 	if (retval != ERROR_OK)
@@ -677,7 +682,7 @@ static int lpcspifi_write(struct flash_bank *bank, const uint8_t *buffer,
 
 	if (target_alloc_working_area(target, sizeof(lpcspifi_flash_write_code),
 			&write_algorithm) != ERROR_OK) {
-		LOG_ERROR("Insufficient working area. You must configure"\
+		LOG_ERROR("Insufficient working area. You must configure"
 			" a working area > %zdB in order to write to SPIFI flash.",
 			sizeof(lpcspifi_flash_write_code));
 		return ERROR_TARGET_RESOURCE_NOT_AVAILABLE;
@@ -699,15 +704,15 @@ static int lpcspifi_write(struct flash_bank *bank, const uint8_t *buffer,
 		 * space, free the algorithm */
 		target_free_working_area(target, write_algorithm);
 
-		LOG_ERROR("Insufficient working area. Please allocate at least"\
+		LOG_ERROR("Insufficient working area. Please allocate at least"
 			" %zdB of working area to enable flash writes.",
 			sizeof(lpcspifi_flash_write_code) + 1
 		);
 
 		return ERROR_TARGET_RESOURCE_NOT_AVAILABLE;
 	} else if (fifo_size < page_size)
-		LOG_WARNING("Working area size is limited; flash writes may be"\
-			" slow. Increase working area size to at least %zdB"\
+		LOG_WARNING("Working area size is limited; flash writes may be"
+			" slow. Increase working area size to at least %zdB"
 			" to reduce write times.",
 			(size_t)(sizeof(lpcspifi_flash_write_code) + page_size)
 		);
@@ -839,11 +844,12 @@ static int lpcspifi_probe(struct flash_bank *bank)
 	struct flash_sector *sectors;
 	uint32_t id = 0; /* silence uninitialized warning */
 	int retval;
+	uint32_t sectorsize;
 
 	/* If we've already probed, we should be fine to skip this time. */
 	if (lpcspifi_info->probed)
 		return ERROR_OK;
-	lpcspifi_info->probed = 0;
+	lpcspifi_info->probed = false;
 
 	lpcspifi_info->ssp_base = 0x40083000;
 	lpcspifi_info->io_base = 0x400F4000;
@@ -876,26 +882,33 @@ static int lpcspifi_probe(struct flash_bank *bank)
 
 	/* Set correct size value */
 	bank->size = lpcspifi_info->dev->size_in_bytes;
+	if (bank->size <= (1UL << 16))
+		LOG_WARNING("device needs 2-byte addresses - not implemented");
+	if (bank->size > (1UL << 24))
+		LOG_WARNING("device needs paging or 4-byte addresses - not implemented");
+
+	/* if no sectors, treat whole bank as single sector */
+	sectorsize = lpcspifi_info->dev->sectorsize ?
+		lpcspifi_info->dev->sectorsize : lpcspifi_info->dev->size_in_bytes;
 
 	/* create and fill sectors array */
-	bank->num_sectors =
-		lpcspifi_info->dev->size_in_bytes / lpcspifi_info->dev->sectorsize;
+	bank->num_sectors = lpcspifi_info->dev->size_in_bytes / sectorsize;
 	sectors = malloc(sizeof(struct flash_sector) * bank->num_sectors);
 	if (sectors == NULL) {
 		LOG_ERROR("not enough memory");
 		return ERROR_FAIL;
 	}
 
-	for (int sector = 0; sector < bank->num_sectors; sector++) {
-		sectors[sector].offset = sector * lpcspifi_info->dev->sectorsize;
-		sectors[sector].size = lpcspifi_info->dev->sectorsize;
+	for (unsigned int sector = 0; sector < bank->num_sectors; sector++) {
+		sectors[sector].offset = sector * sectorsize;
+		sectors[sector].size = sectorsize;
 		sectors[sector].is_erased = -1;
 		sectors[sector].is_protected = 0;
 	}
 
 	bank->sectors = sectors;
 
-	lpcspifi_info->probed = 1;
+	lpcspifi_info->probed = true;
 	return ERROR_OK;
 }
 
@@ -930,7 +943,7 @@ static int get_lpcspifi_info(struct flash_bank *bank, char *buf, int buf_size)
 	return ERROR_OK;
 }
 
-struct flash_driver lpcspifi_flash = {
+const struct flash_driver lpcspifi_flash = {
 	.name = "lpcspifi",
 	.flash_bank_command = lpcspifi_flash_bank_command,
 	.erase = lpcspifi_erase,
@@ -942,4 +955,5 @@ struct flash_driver lpcspifi_flash = {
 	.erase_check = default_flash_blank_check,
 	.protect_check = lpcspifi_protect_check,
 	.info = get_lpcspifi_info,
+	.free_driver_priv = default_flash_free_driver_priv,
 };
