@@ -23,9 +23,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -33,22 +31,37 @@
 #endif
 
 #include <jtag/jtag.h>
+#include <transport/transport.h>
 #include "commands.h"
 
 struct cmd_queue_page {
+	struct cmd_queue_page *next;
 	void *address;
 	size_t used;
-	struct cmd_queue_page *next;
 };
 
 #define CMD_QUEUE_PAGE_SIZE (1024 * 1024)
 static struct cmd_queue_page *cmd_queue_pages;
+static struct cmd_queue_page *cmd_queue_pages_tail;
 
 struct jtag_command *jtag_command_queue;
 static struct jtag_command **next_command_pointer = &jtag_command_queue;
 
 void jtag_queue_command(struct jtag_command *cmd)
 {
+	if (!transport_is_jtag()) {
+		/*
+		 * FIXME: This should not happen!
+		 * There could be old code that queues jtag commands with non jtag interfaces so, for
+		 * the moment simply highlight it by log an error.
+		 * We should fix it quitting with assert(0) because it is an internal error, or returning
+		 * an error after call to jtag_command_queue_reset() to free the jtag queue and avoid
+		 * memory leaks.
+		 * The fix can be applied immediately after next release (v0.11.0 ?)
+		 */
+		LOG_ERROR("JTAG API jtag_queue_command() called on non JTAG interface");
+	}
+
 	/* this command goes on the end, so ensure the queue terminates */
 	cmd->next = NULL;
 
@@ -100,8 +113,7 @@ void *cmd_queue_alloc(size_t size)
 	/* Done... */
 
 	if (*p_page) {
-		while ((*p_page)->next)
-			p_page = &((*p_page)->next);
+		p_page = &cmd_queue_pages_tail;
 		if (CMD_QUEUE_PAGE_SIZE - (*p_page)->used < size)
 			p_page = &((*p_page)->next);
 	}
@@ -113,6 +125,7 @@ void *cmd_queue_alloc(size_t size)
 					CMD_QUEUE_PAGE_SIZE : size;
 		(*p_page)->address = malloc(alloc_size);
 		(*p_page)->next = NULL;
+		cmd_queue_pages_tail = *p_page;
 	}
 
 	offset = (*p_page)->used;
@@ -134,6 +147,7 @@ static void cmd_queue_free(void)
 	}
 
 	cmd_queue_pages = NULL;
+	cmd_queue_pages_tail = NULL;
 }
 
 void jtag_command_queue_reset(void)
@@ -144,6 +158,18 @@ void jtag_command_queue_reset(void)
 	next_command_pointer = &jtag_command_queue;
 }
 
+/**
+ * Copy a struct scan_field for insertion into the queue.
+ *
+ * This allocates a new copy of out_value using cmd_queue_alloc.
+ */
+void jtag_scan_field_clone(struct scan_field *dst, const struct scan_field *src)
+{
+	dst->num_bits	= src->num_bits;
+	dst->out_value	= buf_cpy(src->out_value, cmd_queue_alloc(DIV_ROUND_UP(src->num_bits, 8)), src->num_bits);
+	dst->in_value	= src->in_value;
+}
+
 enum scan_type jtag_scan_type(const struct scan_command *cmd)
 {
 	int i;
@@ -181,33 +207,33 @@ int jtag_build_buffer(const struct scan_command *cmd, uint8_t **buffer)
 
 	bit_count = 0;
 
-	DEBUG_JTAG_IO("%s num_fields: %i",
+	LOG_DEBUG_IO("%s num_fields: %i",
 			cmd->ir_scan ? "IRSCAN" : "DRSCAN",
 			cmd->num_fields);
 
 	for (i = 0; i < cmd->num_fields; i++) {
 		if (cmd->fields[i].out_value) {
-#ifdef _DEBUG_JTAG_IO_
-			char *char_buf = buf_to_str(cmd->fields[i].out_value,
-				(cmd->fields[i].num_bits > DEBUG_JTAG_IOZ)
-					? DEBUG_JTAG_IOZ
-					: cmd->fields[i].num_bits, 16);
+			if (LOG_LEVEL_IS(LOG_LVL_DEBUG_IO)) {
+				char *char_buf = buf_to_hex_str(cmd->fields[i].out_value,
+						(cmd->fields[i].num_bits > DEBUG_JTAG_IOZ)
+						? DEBUG_JTAG_IOZ
+								: cmd->fields[i].num_bits);
 
-			LOG_DEBUG("fields[%i].out_value[%i]: 0x%s", i,
-					cmd->fields[i].num_bits, char_buf);
-			free(char_buf);
-#endif
+				LOG_DEBUG("fields[%i].out_value[%i]: 0x%s", i,
+						cmd->fields[i].num_bits, char_buf);
+				free(char_buf);
+			}
 			buf_set_buf(cmd->fields[i].out_value, 0, *buffer,
 					bit_count, cmd->fields[i].num_bits);
 		} else {
-			DEBUG_JTAG_IO("fields[%i].out_value[%i]: NULL",
+			LOG_DEBUG_IO("fields[%i].out_value[%i]: NULL",
 					i, cmd->fields[i].num_bits);
 		}
 
 		bit_count += cmd->fields[i].num_bits;
 	}
 
-	/*DEBUG_JTAG_IO("bit_count totalling: %i",  bit_count); */
+	/*LOG_DEBUG_IO("bit_count totalling: %i",  bit_count); */
 
 	return bit_count;
 }
@@ -230,16 +256,16 @@ int jtag_read_buffer(uint8_t *buffer, const struct scan_command *cmd)
 			uint8_t *captured = buf_set_buf(buffer, bit_count,
 					malloc(DIV_ROUND_UP(num_bits, 8)), 0, num_bits);
 
-#ifdef _DEBUG_JTAG_IO_
-			char *char_buf = buf_to_str(captured,
-					(num_bits > DEBUG_JTAG_IOZ)
+			if (LOG_LEVEL_IS(LOG_LVL_DEBUG_IO)) {
+				char *char_buf = buf_to_hex_str(captured,
+						(num_bits > DEBUG_JTAG_IOZ)
 						? DEBUG_JTAG_IOZ
-						: num_bits, 16);
+								: num_bits);
 
-			LOG_DEBUG("fields[%i].in_value[%i]: 0x%s",
-					i, num_bits, char_buf);
-			free(char_buf);
-#endif
+				LOG_DEBUG("fields[%i].in_value[%i]: 0x%s",
+						i, num_bits, char_buf);
+				free(char_buf);
+			}
 
 			if (cmd->fields[i].in_value)
 				buf_cpy(captured, cmd->fields[i].in_value, num_bits);
