@@ -32,7 +32,7 @@
 
 #include <jtag/interface.h>
 #include <jtag/commands.h>
-#include "libusb_common.h"
+#include "libusb_helper.h"
 #include <string.h>
 #include <time.h>
 
@@ -134,7 +134,7 @@ static void opendous_tap_append_scan(int length, uint8_t *buffer, struct scan_co
 
 /* opendous lowlevel functions */
 struct opendous_jtag {
-	struct jtag_libusb_device_handle *usb_handle;
+	struct libusb_device_handle *usb_handle;
 };
 
 static struct opendous_jtag *opendous_usb_open(void);
@@ -144,7 +144,7 @@ static int opendous_usb_write(struct opendous_jtag *opendous_jtag, int out_lengt
 static int opendous_usb_read(struct opendous_jtag *opendous_jtag);
 
 /* helper functions */
-int opendous_get_version_info(void);
+static int opendous_get_version_info(void);
 
 #ifdef _DEBUG_USB_COMMS_
 static void opendous_debug_buffer(uint8_t *buffer, int length);
@@ -185,7 +185,7 @@ COMMAND_HANDLER(opendous_handle_opendous_hw_jtag_command)
 {
 	switch (CMD_ARGC) {
 		case 0:
-			command_print(CMD_CTX, "opendous hw jtag  %i", opendous_hw_jtag_version);
+			command_print(CMD, "opendous hw jtag  %i", opendous_hw_jtag_version);
 			break;
 
 		case 1: {
@@ -215,6 +215,7 @@ static const struct command_registration opendous_command_handlers[] = {
 		.handler = &opendous_handle_opendous_info_command,
 		.mode = COMMAND_EXEC,
 		.help = "show opendous info",
+		.usage = "",
 	},
 	{
 		.name = "opendous_hw_jtag",
@@ -233,12 +234,19 @@ static const struct command_registration opendous_command_handlers[] = {
 	COMMAND_REGISTRATION_DONE
 };
 
-struct jtag_interface opendous_interface = {
-	.name = "opendous",
-	.commands = opendous_command_handlers,
+static struct jtag_interface opendous_interface = {
 	.execute_queue = opendous_execute_queue,
+};
+
+struct adapter_driver opendous_adapter_driver = {
+	.name = "opendous",
+	.transports = jtag_only,
+	.commands = opendous_command_handlers,
+
 	.init = opendous_init,
 	.quit = opendous_quit,
+
+	.jtag_ops = &opendous_interface,
 };
 
 static int opendous_execute_queue(void)
@@ -251,7 +259,7 @@ static int opendous_execute_queue(void)
 	while (cmd != NULL) {
 		switch (cmd->type) {
 			case JTAG_RUNTEST:
-				DEBUG_JTAG_IO("runtest %i cycles, end in %i", cmd->cmd.runtest->num_cycles, \
+				LOG_DEBUG_IO("runtest %i cycles, end in %i", cmd->cmd.runtest->num_cycles,
 					cmd->cmd.runtest->end_state);
 
 				if (cmd->cmd.runtest->end_state != -1)
@@ -260,7 +268,7 @@ static int opendous_execute_queue(void)
 				break;
 
 			case JTAG_TLR_RESET:
-				DEBUG_JTAG_IO("statemove end in %i", cmd->cmd.statemove->end_state);
+				LOG_DEBUG_IO("statemove end in %i", cmd->cmd.statemove->end_state);
 
 				if (cmd->cmd.statemove->end_state != -1)
 					opendous_end_state(cmd->cmd.statemove->end_state);
@@ -268,21 +276,21 @@ static int opendous_execute_queue(void)
 				break;
 
 			case JTAG_PATHMOVE:
-				DEBUG_JTAG_IO("pathmove: %i states, end in %i", \
-					cmd->cmd.pathmove->num_states, \
+				LOG_DEBUG_IO("pathmove: %i states, end in %i",
+					cmd->cmd.pathmove->num_states,
 					cmd->cmd.pathmove->path[cmd->cmd.pathmove->num_states - 1]);
 
 				opendous_path_move(cmd->cmd.pathmove->num_states, cmd->cmd.pathmove->path);
 				break;
 
 			case JTAG_SCAN:
-				DEBUG_JTAG_IO("scan end in %i", cmd->cmd.scan->end_state);
+				LOG_DEBUG_IO("scan end in %i", cmd->cmd.scan->end_state);
 
 				if (cmd->cmd.scan->end_state != -1)
 					opendous_end_state(cmd->cmd.scan->end_state);
 
 				scan_size = jtag_build_buffer(cmd->cmd.scan, &buffer);
-				DEBUG_JTAG_IO("scan input, length = %d", scan_size);
+				LOG_DEBUG_IO("scan input, length = %d", scan_size);
 
 #ifdef _DEBUG_USB_COMMS_
 				opendous_debug_buffer(buffer, (scan_size + 7) / 8);
@@ -292,7 +300,7 @@ static int opendous_execute_queue(void)
 				break;
 
 			case JTAG_RESET:
-				DEBUG_JTAG_IO("reset trst: %i srst %i", cmd->cmd.reset->trst, cmd->cmd.reset->srst);
+				LOG_DEBUG_IO("reset trst: %i srst %i", cmd->cmd.reset->trst, cmd->cmd.reset->srst);
 
 				opendous_tap_execute();
 
@@ -302,7 +310,7 @@ static int opendous_execute_queue(void)
 				break;
 
 			case JTAG_SLEEP:
-				DEBUG_JTAG_IO("sleep %" PRIi32, cmd->cmd.sleep->us);
+				LOG_DEBUG_IO("sleep %" PRIu32, cmd->cmd.sleep->us);
 				opendous_tap_execute();
 				jtag_sleep(cmd->cmd.sleep->us);
 				break;
@@ -378,25 +386,17 @@ static int opendous_quit(void)
 {
 	opendous_usb_close(opendous_jtag_handle);
 
-	if (usb_out_buffer) {
-		free(usb_out_buffer);
-		usb_out_buffer = NULL;
-	}
+	free(usb_out_buffer);
+	usb_out_buffer = NULL;
 
-	if (usb_in_buffer) {
-		free(usb_in_buffer);
-		usb_in_buffer = NULL;
-	}
+	free(usb_in_buffer);
+	usb_in_buffer = NULL;
 
-	if (pending_scan_results_buffer) {
-		free(pending_scan_results_buffer);
-		pending_scan_results_buffer = NULL;
-	}
+	free(pending_scan_results_buffer);
+	pending_scan_results_buffer = NULL;
 
-	if (opendous_type) {
-		free(opendous_type);
-		opendous_type = NULL;
-	}
+	free(opendous_type);
+	opendous_type = NULL;
 
 	return ERROR_OK;
 }
@@ -527,7 +527,7 @@ void opendous_simple_command(uint8_t command, uint8_t _data)
 {
 	int result;
 
-	DEBUG_JTAG_IO("0x%02x 0x%02x", command, _data);
+	LOG_DEBUG_IO("0x%02x 0x%02x", command, _data);
 
 	usb_out_buffer[0] = 2;
 	usb_out_buffer[1] = 0;
@@ -544,7 +544,7 @@ int opendous_get_status(void)
 	return ERROR_OK;
 }
 
-int opendous_get_version_info(void)
+static int opendous_get_version_info(void)
 {
 	return ERROR_OK;
 }
@@ -588,7 +588,7 @@ void opendous_tap_append_step(int tms, int tdi)
 		if (!bits)
 			tms_buffer[tap_index] = 0;
 
-		tms_buffer[tap_index] |= (_tdi << bits)|(_tms << (bits + 1)) ;
+		tms_buffer[tap_index] |= (_tdi << bits)|(_tms << (bits + 1));
 		tap_length++;
 	} else
 		LOG_ERROR("opendous_tap_append_step, overflow");
@@ -596,7 +596,7 @@ void opendous_tap_append_step(int tms, int tdi)
 
 void opendous_tap_append_scan(int length, uint8_t *buffer, struct scan_command *command)
 {
-	DEBUG_JTAG_IO("append scan, length = %d", length);
+	LOG_DEBUG_IO("append scan, length = %d", length);
 
 	struct pending_scan_result *pending_scan_result = &pending_scan_results_buffer[pending_scan_results_length];
 	int i;
@@ -678,7 +678,7 @@ int opendous_tap_execute(void)
 			/* Copy to buffer */
 			buf_set_buf(tdo_buffer, first, buffer, 0, length);
 
-			DEBUG_JTAG_IO("pending scan result, length = %d", length);
+			LOG_DEBUG_IO("pending scan result, length = %d", length);
 
 #ifdef _DEBUG_USB_COMMS_
 			opendous_debug_buffer(buffer, byte_length_out);
@@ -689,8 +689,7 @@ int opendous_tap_execute(void)
 				return ERROR_JTAG_QUEUE_FAILED;
 			}
 
-			if (pending_scan_result->buffer != NULL)
-				free(pending_scan_result->buffer);
+			free(pending_scan_result->buffer);
 		}
 
 		opendous_tap_init();
@@ -706,12 +705,12 @@ struct opendous_jtag *opendous_usb_open(void)
 {
 	struct opendous_jtag *result;
 
-	struct jtag_libusb_device_handle *devh;
-	if (jtag_libusb_open(opendous_probe->VID, opendous_probe->PID, NULL, &devh) != ERROR_OK)
+	struct libusb_device_handle *devh;
+	if (jtag_libusb_open(opendous_probe->VID, opendous_probe->PID, NULL, &devh, NULL) != ERROR_OK)
 		return NULL;
 
 	jtag_libusb_set_configuration(devh, 0);
-	jtag_libusb_claim_interface(devh, 0);
+	libusb_claim_interface(devh, 0);
 
 	result = malloc(sizeof(*result));
 	result->usb_handle = devh;
@@ -762,14 +761,14 @@ int opendous_usb_write(struct opendous_jtag *opendous_jtag, int out_length)
 			LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE | LIBUSB_ENDPOINT_OUT,
 			FUNC_WRITE_DATA, 0, 0, (char *) usb_out_buffer, out_length, OPENDOUS_USB_TIMEOUT);
 	} else {
-		result = jtag_libusb_bulk_write(opendous_jtag->usb_handle, OPENDOUS_WRITE_ENDPOINT, \
-			(char *)usb_out_buffer, out_length, OPENDOUS_USB_TIMEOUT);
+		jtag_libusb_bulk_write(opendous_jtag->usb_handle, OPENDOUS_WRITE_ENDPOINT,
+			(char *)usb_out_buffer, out_length, OPENDOUS_USB_TIMEOUT, &result);
 	}
 #ifdef _DEBUG_USB_COMMS_
 	LOG_DEBUG("USB write end: %d bytes", result);
 #endif
 
-	DEBUG_JTAG_IO("opendous_usb_write, out_length = %d, result = %d", out_length, result);
+	LOG_DEBUG_IO("opendous_usb_write, out_length = %d, result = %d", out_length, result);
 
 #ifdef _DEBUG_USB_COMMS_
 	opendous_debug_buffer(usb_out_buffer, out_length);
@@ -789,13 +788,13 @@ int opendous_usb_read(struct opendous_jtag *opendous_jtag)
 			LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE | LIBUSB_ENDPOINT_IN,
 			FUNC_READ_DATA, 0, 0, (char *) usb_in_buffer, OPENDOUS_IN_BUFFER_SIZE, OPENDOUS_USB_TIMEOUT);
 	} else {
-		result = jtag_libusb_bulk_read(opendous_jtag->usb_handle, OPENDOUS_READ_ENDPOINT,
-			(char *)usb_in_buffer, OPENDOUS_IN_BUFFER_SIZE, OPENDOUS_USB_TIMEOUT);
+		jtag_libusb_bulk_read(opendous_jtag->usb_handle, OPENDOUS_READ_ENDPOINT,
+			(char *)usb_in_buffer, OPENDOUS_IN_BUFFER_SIZE, OPENDOUS_USB_TIMEOUT, &result);
 	}
 #ifdef _DEBUG_USB_COMMS_
 	LOG_DEBUG("USB read end: %d bytes", result);
 #endif
-	DEBUG_JTAG_IO("opendous_usb_read, result = %d", result);
+	LOG_DEBUG_IO("opendous_usb_read, result = %d", result);
 
 #ifdef _DEBUG_USB_COMMS_
 	opendous_debug_buffer(usb_in_buffer, result);
