@@ -35,7 +35,7 @@
 
 #include <target/target.h>
 
-static struct hl_interface_s hl_if = { {0, 0, 0, 0, 0, HL_TRANSPORT_UNKNOWN, false, -1}, 0, 0 };
+static struct hl_interface_s hl_if = { {0, 0, { 0 }, { 0 }, HL_TRANSPORT_UNKNOWN, false, -1}, 0, 0 };
 
 int hl_interface_open(enum hl_transports tr)
 {
@@ -119,25 +119,27 @@ static int hl_interface_quit(void)
 	if (hl_if.layout->api->close)
 		hl_if.layout->api->close(hl_if.handle);
 
+	jtag_command_queue_reset();
+
+	free((void *)hl_if.param.device_desc);
+	free((void *)hl_if.param.serial);
+
 	return ERROR_OK;
 }
 
-static int hl_interface_execute_queue(void)
+static int hl_interface_reset(int req_trst, int req_srst)
 {
-	LOG_DEBUG("hl_interface_execute_queue: ignored");
-
-	return ERROR_OK;
+	return hl_if.layout->api->assert_srst(hl_if.handle, req_srst ? 0 : 1);
 }
 
 int hl_interface_init_reset(void)
 {
-	/* incase the adapter has not already handled asserting srst
+	/* in case the adapter has not already handled asserting srst
 	 * we will attempt it again */
 	if (hl_if.param.connect_under_reset) {
-		jtag_add_reset(0, 1);
-		hl_if.layout->api->assert_srst(hl_if.handle, 0);
+		adapter_assert_reset();
 	} else {
-		jtag_add_reset(0, 0);
+		adapter_deassert_reset();
 	}
 
 	return ERROR_OK;
@@ -186,12 +188,13 @@ int hl_interface_override_target(const char **targetname)
 	return ERROR_FAIL;
 }
 
-int hl_interface_config_trace(bool enabled, enum tpio_pin_protocol pin_protocol,
-			      uint32_t port_size, unsigned int *trace_freq)
+static int hl_interface_config_trace(bool enabled, enum tpiu_pin_protocol pin_protocol,
+		uint32_t port_size, unsigned int *trace_freq,
+		unsigned int traceclkin_freq, uint16_t *prescaler)
 {
 	if (hl_if.layout->api->config_trace)
-		return hl_if.layout->api->config_trace(hl_if.handle, enabled, pin_protocol,
-						       port_size, trace_freq);
+		return hl_if.layout->api->config_trace(hl_if.handle, enabled,
+			pin_protocol, port_size, trace_freq, traceclkin_freq, prescaler);
 	else if (enabled) {
 		LOG_ERROR("The selected interface does not support tracing");
 		return ERROR_FAIL;
@@ -200,7 +203,7 @@ int hl_interface_config_trace(bool enabled, enum tpio_pin_protocol pin_protocol,
 	return ERROR_OK;
 }
 
-int hl_interface_poll_trace(uint8_t *buf, size_t *size)
+static int hl_interface_poll_trace(uint8_t *buf, size_t *size)
 {
 	if (hl_if.layout->api->poll_trace)
 		return hl_if.layout->api->poll_trace(hl_if.handle, buf, size);
@@ -264,15 +267,27 @@ COMMAND_HANDLER(hl_interface_handle_layout_command)
 
 COMMAND_HANDLER(hl_interface_handle_vid_pid_command)
 {
-	LOG_DEBUG("hl_interface_handle_vid_pid_command");
-
-	if (CMD_ARGC != 2) {
-		LOG_WARNING("ignoring extra IDs in hl_vid_pid (maximum is 1 pair)");
+	if (CMD_ARGC > HLA_MAX_USB_IDS * 2) {
+		LOG_WARNING("ignoring extra IDs in hla_vid_pid "
+			"(maximum is %d pairs)", HLA_MAX_USB_IDS);
+		CMD_ARGC = HLA_MAX_USB_IDS * 2;
+	}
+	if (CMD_ARGC < 2 || (CMD_ARGC & 1)) {
+		LOG_WARNING("incomplete hla_vid_pid configuration directive");
 		return ERROR_COMMAND_SYNTAX_ERROR;
 	}
 
-	COMMAND_PARSE_NUMBER(u16, CMD_ARGV[0], hl_if.param.vid);
-	COMMAND_PARSE_NUMBER(u16, CMD_ARGV[1], hl_if.param.pid);
+	unsigned i;
+	for (i = 0; i < CMD_ARGC; i += 2) {
+		COMMAND_PARSE_NUMBER(u16, CMD_ARGV[i], hl_if.param.vid[i / 2]);
+		COMMAND_PARSE_NUMBER(u16, CMD_ARGV[i + 1], hl_if.param.pid[i / 2]);
+	}
+
+	/*
+	 * Explicitly terminate, in case there are multiple instances of
+	 * hla_vid_pid.
+	 */
+	hl_if.param.vid[i / 2] = hl_if.param.pid[i / 2] = 0;
 
 	return ERROR_OK;
 }
@@ -297,7 +312,7 @@ static const struct command_registration hl_interface_command_handlers[] = {
 	 .name = "hla_device_desc",
 	 .handler = &hl_interface_handle_device_desc_command,
 	 .mode = COMMAND_CONFIG,
-	 .help = "set the a device description of the adapter",
+	 .help = "set the device description of the adapter",
 	 .usage = "description_string",
 	 },
 	{
@@ -326,22 +341,24 @@ static const struct command_registration hl_interface_command_handlers[] = {
 	 .handler = &interface_handle_hla_command,
 	 .mode = COMMAND_EXEC,
 	 .help = "execute a custom adapter-specific command",
-	 .usage = "hla_command <command>",
+	 .usage = "<command>",
 	 },
 	COMMAND_REGISTRATION_DONE
 };
 
-struct jtag_interface hl_interface = {
+struct adapter_driver hl_adapter_driver = {
 	.name = "hla",
-	.supported = 0,
-	.commands = hl_interface_command_handlers,
 	.transports = hl_transports,
+	.commands = hl_interface_command_handlers,
+
 	.init = hl_interface_init,
 	.quit = hl_interface_quit,
-	.execute_queue = hl_interface_execute_queue,
+	.reset = hl_interface_reset,
 	.speed = &hl_interface_speed,
 	.khz = &hl_interface_khz,
 	.speed_div = &hl_interface_speed_div,
 	.config_trace = &hl_interface_config_trace,
 	.poll_trace = &hl_interface_poll_trace,
+
+	/* no ops for HLA, targets hla_target and stm8 intercept them all */
 };
