@@ -36,6 +36,29 @@
  * may be separate registers associated with debug or trace modules.
  */
 
+struct reg *register_get_by_number(struct reg_cache *first,
+		uint32_t reg_num, bool search_all)
+{
+	unsigned i;
+	struct reg_cache *cache = first;
+
+	while (cache) {
+		for (i = 0; i < cache->num_regs; i++) {
+			if (cache->reg_list[i].exist == false)
+				continue;
+			if (cache->reg_list[i].number == reg_num)
+				return &(cache->reg_list[i]);
+		}
+
+		if (search_all)
+			cache = cache->next;
+		else
+			break;
+	}
+
+	return NULL;
+}
+
 struct reg *register_get_by_name(struct reg_cache *first,
 		const char *name, bool search_all)
 {
@@ -44,6 +67,8 @@ struct reg *register_get_by_name(struct reg_cache *first,
 
 	while (cache) {
 		for (i = 0; i < cache->num_regs; i++) {
+			if (cache->reg_list[i].exist == false)
+				continue;
 			if (strcmp(cache->reg_list[i].name, name) == 0)
 				return &(cache->reg_list[i]);
 		}
@@ -84,8 +109,10 @@ void register_cache_invalidate(struct reg_cache *cache)
 	struct reg *reg = cache->reg_list;
 
 	for (unsigned n = cache->num_regs; n != 0; n--, reg++) {
-		reg->valid = 0;
-		reg->dirty = 0;
+		if (reg->exist == false)
+			continue;
+		reg->valid = false;
+		reg->dirty = false;
 	}
 }
 
@@ -96,8 +123,8 @@ static int register_get_dummy_core_reg(struct reg *reg)
 
 static int register_set_dummy_core_reg(struct reg *reg, uint8_t *buf)
 {
-	reg->dirty = 1;
-	reg->valid = 1;
+	reg->dirty = true;
+	reg->valid = true;
 
 	return ERROR_OK;
 }
