@@ -19,6 +19,10 @@
 #ifndef OPENOCD_TRANSPORT_TRANSPORT_H
 #define OPENOCD_TRANSPORT_TRANSPORT_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "helper/command.h"
 
 /**
@@ -88,6 +92,19 @@ COMMAND_HELPER(transport_list_parse, char ***vector);
 
 int allow_transports(struct command_context *ctx, const char * const *vector);
 
-bool transports_are_declared(void);
+bool transport_is_jtag(void);
+bool transport_is_swd(void);
+bool transport_is_dapdirect_jtag(void);
+bool transport_is_dapdirect_swd(void);
+bool transport_is_swim(void);
+
+#if BUILD_HLADAPTER && !HAVE_JTAG_MINIDRIVER_H
+bool transport_is_hla(void);
+#else
+static inline bool transport_is_hla(void)
+{
+	return false;
+}
+#endif
 
 #endif /* OPENOCD_TRANSPORT_TRANSPORT_H */
